@@ -4,42 +4,31 @@ import logging
 import os
 import pickle
 import docker.errors
+import json
 
-from enum import Enum
 from typing import Dict, List, Union
 from pathlib import Path
 
 from vantage6.common.globals import APPNAME
 from vantage6.common.docker.addons import (
-    remove_container_if_exists, remove_container
+    remove_container_if_exists, remove_container, pull_if_newer,
+    running_in_docker
 )
-from vantage6.node.util import logger_name
+from vantage6.common.docker.network_manager import NetworkManager
+from vantage6.common.task_status import TaskStatus
+from vantage6.node.util import logger_name, get_parent_id
 from vantage6.node.globals import ALPINE_IMAGE
 from vantage6.node.docker.vpn_manager import VPNManager
-from vantage6.common.docker.network_manager import NetworkManager
 from vantage6.node.docker.docker_base import DockerBaseManager
-from vantage6.common.docker.addons import pull_if_newer, running_in_docker
 from vantage6.node.docker.exceptions import (
     UnknownAlgorithmStartFail,
     PermanentAlgorithmStartFail
 )
 
-class TaskStatus(Enum):
-    # Task constructor is executed
-    INITIALIZED = 0
-    # Container started without exceptions
-    STARTED = 1
-    # Container exited and had zero exit code
-    # COMPLETED = 2
-    # Failed to start the container
-    FAILED = 90
-    PERMANENTLY_FAILED = 91
-    # Container had a non zero exit code
-    # CRASHED = 92
 
 class DockerTaskManager(DockerBaseManager):
     """
-    Manager for running a Vantage6 algorithm container within docker.
+    Manager for running a vantage6 algorithm container within docker.
 
     Ensures that the environment is properly set up (docker volumes,
     directories, environment variables, etc). Then runs the algorithm as a
@@ -49,7 +38,7 @@ class DockerTaskManager(DockerBaseManager):
     log = logging.getLogger(logger_name(__name__))
 
     def __init__(self, image: str, vpn_manager: VPNManager, node_name: str,
-                 result_id: int, tasks_dir: Path,
+                 result_id: int, task_info: Dict, tasks_dir: Path,
                  isolated_network_mgr: NetworkManager,
                  databases: dict, docker_volume_name: str,
                  alpine_image: Union[str, None] = None):
@@ -66,6 +55,8 @@ class DockerTaskManager(DockerBaseManager):
             Name of the node, to track running algorithms
         result_id: int
             Server result identifier
+        task_info: Dict
+            Dictionary with info about the task
         tasks_dir: Path
             Directory in which this task's data are stored
         isolated_network_mgr: NetworkManager
@@ -81,6 +72,8 @@ class DockerTaskManager(DockerBaseManager):
         self.image = image
         self.__vpn_manager = vpn_manager
         self.result_id = result_id
+        self.task_id = task_info['id']
+        self.parent_id = get_parent_id(task_info)
         self.__tasks_dir = tasks_dir
         self.databases = databases
         self.data_volume_name = docker_volume_name
@@ -106,7 +99,7 @@ class DockerTaskManager(DockerBaseManager):
         self.data_folder = "/mnt/data"
 
         # keep track of the task status
-        self.status: TaskStatus = TaskStatus.INITIALIZED
+        self.status: TaskStatus = TaskStatus.INITIALIZING
 
     def is_finished(self) -> bool:
         """
@@ -138,6 +131,9 @@ class DockerTaskManager(DockerBaseManager):
             self.log.error(f"Received non-zero exitcode: {self.status_code}")
             self.log.error(f"  Container id: {self.container.id}")
             self.log.info(logs)
+            self.status = TaskStatus.CRASHED
+        else:
+            self.status = TaskStatus.COMPLETED
         return logs
 
     def get_results(self) -> bytes:
@@ -159,9 +155,16 @@ class DockerTaskManager(DockerBaseManager):
             self.log.info(f"Retrieving latest image: '{self.image}'")
             pull_if_newer(self.docker, self.image, self.log)
 
+        except docker.errors.APIError as e:
+            self.log.debug('Failed to pull image: could not find image')
+            self.log.exception(e)
+            self.status = TaskStatus.NO_DOCKER_IMAGE
+            raise PermanentAlgorithmStartFail
         except Exception as e:
             self.log.debug('Failed to pull image')
-            self.log.error(e)
+            self.log.exception(e)
+            self.status = TaskStatus.FAILED
+            raise PermanentAlgorithmStartFail
 
     def run(self, docker_input: bytes, tmp_vol_name: str, token: str,
             algorithm_env: Dict, database: str) -> List[Dict]:
@@ -283,16 +286,11 @@ class DockerTaskManager(DockerBaseManager):
                 labels=self.labels
             )
 
-        except docker.errors.ImageNotFound:
-            self.log.error(f'Could not find image: {self.image}')
-            self.status = TaskStatus.PERMANENTLY_FAILED
-            raise PermanentAlgorithmStartFail
-
         except Exception as e:
-            self.status = TaskStatus.FAILED
+            self.status = TaskStatus.START_FAILED
             raise UnknownAlgorithmStartFail(e)
 
-        self.status = TaskStatus.STARTED
+        self.status = TaskStatus.ACTIVE
         return vpn_ports
 
     def _make_task_folders(self) -> None:
@@ -402,16 +400,20 @@ class DockerTaskManager(DockerBaseManager):
         }
 
         # Only prepend the data_folder is it is a file-based database
-        # This allows algorithms to access multiple data-sources at the
+        # This allows algorithms to access multiple data sources at the
         # same time
+        db_labels = []
         for label in self.databases:
             db = self.databases[label]
             var_name = f'{label.upper()}_DATABASE_URI'
             environment_variables[var_name] = \
                 f"{self.data_folder}/{os.path.basename(db['uri'])}" \
                 if db['is_file'] else db['uri']
+            db_labels.append(label)
+        environment_variables['DB_LABELS'] = json.dumps(db_labels)
 
         # Support legacy algorithms
+        # TODO remove in v4+
         try:
             environment_variables["DATABASE_URI"] = (
                 f"{self.data_folder}/"
